#include <iostream>
#include <vector>
#include <memory>

#include "direct_binding.h"

/**
* Visualization
*/
#include "VideoDisplay.h"

#define N_CAMS 2

VideoDisplay* _displays[N_CAMS];
HANDLE _camWorkerThreadHandles[N_CAMS];
HANDLE _displayThreadHandles[N_CAMS];
unsigned int _camIds[N_CAMS];

bool _runs[N_CAMS];


DWORD WINAPI camWorker(LPVOID lpParam)
{
  unsigned int i = (unsigned int)lpParam;
  int w;
  int h;

  if (::evo_irimager_multi_get_palette_image_size(_camIds[i], &w, &h) == 0)
  {
    std::vector<unsigned char> img(w * h * 3);
    while (_runs[i])
    {
      if (::evo_irimager_multi_get_palette_image(_camIds[i], &w, &h, &img[0]) == 0)
      {
        if (_displays[i])
          _displays[i]->drawCapture(0, 0, w, h, 24, &img[0]);
      }
    }
  }

	return 0;
}

int main(int argc, char *argv[])
{
  if(argc< (N_CAMS + 1))
  {
    std::cout << "usage: " << argv[0];
         
    for (int i = 0; i < N_CAMS; i++)
    {
      std::cout << " <configuration xml " << i << " >";
    }
    std::cout << std::endl;
    return -1;
  }




  for (int i = 0; i < N_CAMS; i++)
  {

    int ret = ::evo_irimager_multi_usb_init(&_camIds[i], argv[1+i], 0, 0);
    if (ret != 0)
    {
      std::cerr << "error at init" << std::endl;
      std::cin.ignore();
      return -1;
    }

    int w;
    int h;

    if (::evo_irimager_multi_get_palette_image_size(_camIds[i], &w, &h) == 0)
    {
      _runs[i] = true;
      if (w < 120) w *= 2;
      if (h < 120) h *= 2;
      _displays[i] = new VideoDisplay(w, h);
      _displayThreadHandles[i] = _displays[i]->showDetach();
      _camWorkerThreadHandles[i] = CreateThread(NULL, 0, camWorker, (LPVOID)i, 0, NULL);
    }
    else {
      std::cerr << "error at get_palette_image_size" << std::endl;
    }
  }

  for (int i = 0; i < N_CAMS; i++)
  {
    WaitForSingleObject(_displayThreadHandles[i], INFINITE);
    _runs[i] = false;
    WaitForSingleObject(_camWorkerThreadHandles[i], INFINITE);
    CloseHandle(_displayThreadHandles[i]);
    CloseHandle(_camWorkerThreadHandles[i]);
    evo_irimager_multi_terminate(_camIds[i]);
  }

  return 0;
}
